#ifndef XSV95_H
#define XSV95_H

#include "SRECRD.H"
#include "XC95.H"

class XSV95 : public XC95
{
	public:

	/* begin XSV95 methods */

	/* method: XSV95 */
	// instantiate an object for loading an XSV Board
	XSV95(XSError* err, unsigned int portNum, string& intfcDir);

	/* method: ConfigureFlashInterface */
	// configure the XC95108 on the XSV for programming the Flash RAM
	bool ConfigureFlashInterface(void);

	/* method: ConfigureDefaultFlashConfigInterface */
	// configure the XC95108 on the XSV for programming the Virtex from the Flash RAM
	bool ConfigureDefaultFlashConfigInterface(void);

	/* method: DownloadFlash */
	// download the XSV Flash with the contents of an EXO file
	bool DownloadFlash(string& exofileName);

	/* method: DownloadFlash */
	// download the XSV Flash with the contents arriving through a stream
	bool DownloadFlash(istream& is);

	/* method: DownloadHexRecordToFlash */
	// load RAM data at the addresses given in a hex record
	void DownloadHexRecordToFlash(SRecord& hx);

	/* method: UploadFlash */
	// upload the XSV Flash to a file
	bool UploadFlash(
		string& exofileName, // dump uploaded data to this file
		unsigned long loAddr, // start fetching data from this address
		unsigned long hiAddr // stop at this address
		);

	/* method: UploadFlash */
	// upload the XSV Flash to a stream
	bool UploadFlash(
		ostream& os, // dump uploaded data to this stream
		unsigned long loAddr, // start fetching data from this address
		unsigned long hiAddr // stop at this address
		);

	/* method: UploadHexRecordFromFlash */
	// get Flash data at the addresses given and store it in a hex record
	void UploadHexRecordFromFlash(
		SRecord& hx, // hex record to store data in
		unsigned long loAddr, // begin upload from this address
		unsigned long hiAddr // end upload at this address
		);

	/* method: ReadFlashByte */
	// read a byte from the Flash through the parallel port
	unsigned int ReadFlashByte(unsigned int address);

	/* method: WriteFlashByte */
	// write a byte to the Flash through the parallel port
	void WriteFlashByte(unsigned int address, unsigned int data);

	/* method: ResetFlash */
	// reset the Flash so data can be read from it
	void ResetFlash();

	/* method: ReadFlashID */
	// read data from the Flash
	unsigned int ReadFlashID();

	/* method: ReadFlashStatus */
	// get the status of the Flash
	unsigned int ReadFlashStatus();

	/* method: ClearFlashStatus */
	// clear any error bits in the Flash status register
	void ClearFlashStatus();

	/* method: LockFlashBlock */
	void LockFlashBlock(unsigned int address);

	/* method: MasterLockFlash */
	// lock all the block lock bits so they cannot be altered
	void MasterLockFlash();

	/* method: UnlockFlashBlocks */
	void UnlockFlashBlocks();

	/* method: ProgramFlash */
	// Program a byte in the Flash
	void ProgramFlash(unsigned int address, unsigned int data);

	/* method: EraseFlashBlock */
	// erase the contents of a Flash block
	void EraseFlashBlock(unsigned int address);

	/* end XSV95 methods */

	private:
	bool flashInterfaceIsRdy;
	string flashInterfaceDir;
};

/* begin XSV95 functions */

/* end XSV95 functions */

#endif
