$MOD51
$TITLE(XS Board Hardware Tester)

YES             EQU     1
NO              EQU     0

STACKTOP EQU    70H    ; start of stack (grows up)
LEDLATCH EQU    0FFFFH ; LED latch address

OKLED           EQU     01110111B       ; code for 'O'
ERRORLED        EQU     01101101B       ; code for 'E'
SEED			EQU		1				; RANDOM NUMBER SEED
HEARTBEAT		EQU		P3.1			; INFORMS PC WE ARE ALIVE
RANDBIT			EQU		0				; STORAGE FOR RANDOM BIT
MEMORYEND		EQU		080H			; FIRST NONEXISTENT RAM LOCATION

        DSEG AT 031H
RAND:			DS      1
ERRFLAG:		DS		1				; STORAGE FOR ERROR FLAG
PROGRESS:		DS      1

        CSEG
        ORG     0H

        MOV     IE,#0
        MOV     SP,#STACKTOP
        MOV     PROGRESS,#1
        MOV     A,PROGRESS
        CALL    SHOWLED
        MOV     RAND,#SEED
        MOV     DPTR,#MEMORYSTART
LOAD_MEM:
        CALL    RNG
        MOVX    @DPTR,A
        INC     DPTR
        MOV     A,DPL
        JNZ     LOAD_MEM 
		CPL		HEARTBEAT
        MOV     A,PROGRESS
        RL      A
        MOV     PROGRESS,A
        CALL    SHOWLED
        MOV     A,DPH
        CJNE    A,#MEMORYEND,LOAD_MEM         

		MOV		ERRFLAG,#NO
        MOV     RAND,#SEED
        MOV     DPTR,#MEMORYSTART
CHECK_MEM:
        CALL    RNG
        MOV     B,ACC
        MOVX    A,@DPTR
        CLR     C
        SUBB    A,B
        JZ      CHECK_MEM1
        MOV		ERRFLAG,#YES
CHECK_MEM1:
        INC     DPTR
        MOV     A,DPL
        JNZ     CHECK_MEM
		CPL		HEARTBEAT
        MOV     A,DPH
        CJNE    A,#MEMORYEND,CHECK_MEM        

		MOV		A,ERRFLAG
        JZ		MEM_OK     
        MOV     A,#ERRORLED
        CALL    SHOWLED
        JMP     $				; ERROR - STOP HEARTBEAT
MEM_OK:
        MOV     A,#OKLED
        CALL    SHOWLED
MEM_OK1:
		CPL		HEARTBEAT		; OK - OUTPUT RAPID HEARTBEAT
        JMP		MEM_OK1     


; GENERATE A NEW RANDOM NUMBER AND RETURN IT IN ACC
RNG:
		CLR		RANDBIT	; INITIALIZE NEXT RANDOM BIT
		MOV		A,RAND	; GET PREVIOUS RANDOM NUMBER
		RRC		A		; GET BIT0 OF RANDOM NUMBER
		JNC		RNG1	; IF BIT0=0, DO NOT XOR WITH RANDOM BIT
		CPL		RANDBIT	; OTHERWISE XOR WITH RANDOM BIT
RNG1:
		RRC		A		; GET BIT2 OF RANDOM NUMBER
		RRC		A
		JNC		RNG2	; IF BIT2=0, DO NOT XOR WITH RANDOM BIT
		CPL		RANDBIT	; OTHERWISE XOR WITH RANDOM BIT
RNG2:
		RRC		A		; GET BIT3 OF RANDOM NUMBER
		JNC		RNG3	; AGAIN, XOR ONLY IF BIT3=1
		CPL		RANDBIT
RNG3:
		RRC		A		; GET BIT4 OF RANDOM NUMBER
		JNC		RNG4	; AGAIN, XOR ONLY IF BIT4=1
		CPL		RANDBIT
RNG4:
		MOV		A,RAND	; RESTORE PREVIOUS RANDOM NUMBER
		MOV		C,RANDBIT	; LOAD NEW RANDOM BIT INTO CARRY
		RRC		A		; SHIFT NEW RANDOM BIT INTO MSB OF RANDOM NUMBER
		MOV		RAND,A	; STORE NEW RANDOM NUMBER
        RET				; RETURN WITH NEW RANDOM NUMBER IN ACC

; SHOW THE BIT PATTERN IN THE ACC ON THE 7-SEGMENT LED
SHOWLED:
        PUSH    DPH
        PUSH    DPL
        MOV     DPTR,#LEDLATCH
        MOVX    @DPTR,A
        POP     DPL
        POP     DPH
        RET
        NOP
        NOP

MEMORYSTART     EQU     $

        END
